import { Server as HTTPServer } from "http";
import { Server as HTTPSServer } from "https";
import { Http2SecureServer, Http2Server } from "http2";

/**
 * The collaboration server, provides bidirectional communication, middleware and hooks mechanism.
 */
export declare class Server {
    constructor(config?: IServerConfig);

    /**
     * Register middlewares to the server
     * @param middlewares - middlewares to register
     */
    use(middlewares: IMiddlewares): void;
    /**
     * Register middleware to the server
     * @param action - action name
     * @param middleware - middleware to register
     */
    use<K extends keyof IMiddlewareContext>(action: K, middleware: IMiddleware<IMiddlewareContext[K]>): void;

    /**
     * Register hooks to the server
     * @param hooks - hooks to register
     */
    on(hooks: IHooks): void;
    /**
     * Register hooks to the server
     * @param action - action name
     * @param hook - hook to register
     */
    on<K extends keyof IHookContext>(action: K, hook: IHook<IHookContext[K]>): void;

    /**
     * Register feature to the server
     * @param feature - feature to register
     */
    useFeature(feature: IFeature): void;
}

export interface IServerConfig {
    httpServer?: HTTPServer | HTTPSServer | Http2SecureServer | Http2Server;
    port?: number;
    /**
     * The request path of the server. default is '/collaboration/'
     */
    path?: string;
}

export interface IMiddleware<T> {
    (context: T, next: INext): Promise<void>;
}

export interface IMiddlewares {
    connect?: IMiddleware<IConnectMiddlewareContext>,
    message?: IMiddleware<IMessageMiddlewareContext>
}

export interface IMiddlewareContext {
    connect: IConnectMiddlewareContext,
    message: IMessageMiddlewareContext
}

export interface IConnectMiddlewareContext {
    connection: Connection;
    roomId: string;
}

export interface IMessageMiddlewareContext {
    roomId: string;
    connection: Connection;
    type: MessageType;
    data: MessageData;
}

export type MessageType = string | null;
export type MessageData = unknown;

export interface INext {
    (error?: Error | string): Promise<void>;
}

export interface IHook<T> {
    (context: T): void;
}
export interface IHooks {
    connect?: IHook<IConnectContext>,
    message?: IHook<IMessageContext>,
    createRoom?: IHook<ICreateRoomContext>,
    disconnect?: IHook<IDisconnectContext>,
    enterRoom?: IHook<IEnterRoomContext>,
    leaveRoom?: IHook<ILeaveRoomContext>,
    destroyRoom?: IHook<IDestroyRoomContext>,
}

export interface IHookContext {
    connect: IConnectContext,
    createRoom: ICreateRoomContext,
    enterRoom: IEnterRoomContext,
    message: IMessageContext,
    disconnect: IDisconnectContext,
    leaveRoom: ILeaveRoomContext,
    destroyRoom: IDestroyRoomContext,
}

export interface IConnectContext {
    roomId: string;
    connection: Connection;
}

export interface IMessageContext {
    roomId: string;
    connection: Connection;
    type: MessageType;
    data: MessageData;
}

export interface ICreateRoomContext {
    roomId: string;
}

export interface IEnterRoomContext {
    roomId: string;
    connection: Connection;
}

export interface IDisconnectContext {
    roomId: string;
    connection: Connection;
}

export interface ILeaveRoomContext {
    roomId: string;
    connection: Connection;
}

export interface IDestroyRoomContext {
    roomId: string;
}

export interface IFeature {
    middlewares?: IMiddlewares;
    hooks?: IHooks;
}

/**
 * The collaboration connection of server.
 */
export declare class Connection {
    /**
     * Save custom data for each connection.
     */
    tags: Map<string, unknown>;
    /**
     * Get the id of the connection.
     * @returns {string} Returns the id of the connection.
     */
    get id(): string;
    /**
     * The room id of the connection.
     * @returns {string} Returns the room id of the connection.
     */
    get roomId(): string;
    /**
     * Get the query arguments of the connection.
     * @returns {Record<string, unknown>} Returns the query arguments of the connection.
     */
    get query(): Record<string, unknown>;
    /**
     * Get the authentication data of the connection.
     * @returns {[key: string]: any} Returns the authentication data of the connection.
     */
    get auth(): {
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        [key: string]: any;
    };
    /**
     * Send message to client
     * @param data - message data to send
     * @param type - message type
     */
    send(data: MessageData, type?: MessageType): void;
    /**
     * Broadcast message to all connections in the room
     * @param data - message data to send
     * @param type - message type
     * @param includeSelf - whether to send to self
     */
    broadcast(data: string | Uint8Array, type: MessageType, includeSelf?: boolean): void;
    /**
     * Close the connection
     */
    close(): void;
}